@@ -2,8 +2,9 @@
 # Addition of Martuzzi and Elliott Copyright 2006 Olaf Berke and Roger Bivand
 #
 
-EBImoran <- function (z, listw, nn, S0, zero.policy = NULL, subtract_mean_in_numerator=FALSE) 
+EBImoran <- function (z, listw, nn, S0, zero.policy = attr(listw, "zero.policy"), subtract_mean_in_numerator=TRUE) 
 {
+#default subtract_mean_in_numerator=TRUE 160219 RA
         if (is.null(zero.policy))
             zero.policy <- get("zeroPolicy", envir = .spdepOptions)
         stopifnot(is.logical(zero.policy))
@@ -17,13 +18,15 @@ EBImoran <- function (z, listw, nn, S0, zero.policy = NULL, subtract_mean_in_num
     res
 }
 
-EBImoran.mc <- function (n, x, listw, nsim, zero.policy = NULL,
- alternative = "greater", spChk = NULL, return_boot=FALSE, subtract_mean_in_numerator=FALSE) 
+EBImoran.mc <- function (n, x, listw, nsim, zero.policy = attr(listw, "zero.policy"),
+ alternative = "greater", spChk = NULL, return_boot=FALSE, subtract_mean_in_numerator=TRUE) 
 {
+#default subtract_mean_in_numerator=TRUE 160219 RA
+    message("The default for subtract_mean_in_numerator set TRUE from February 2016")
         if (is.null(zero.policy))
             zero.policy <- get("zeroPolicy", envir = .spdepOptions)
         stopifnot(is.logical(zero.policy))
-    alternative <- match.arg(alternative, c("greater", "less"))
+    alternative <- match.arg(alternative, c("greater", "less", "two.sided"))
     if (!inherits(listw, "listw")) 
         stop(paste(deparse(substitute(listw)), "is not a listw object"))
     if (missing(nsim)) 
@@ -55,21 +58,10 @@ EBImoran.mc <- function (n, x, listw, nsim, zero.policy = NULL,
             var <- var[i]
             return(EBImoran(z=var, ...))
         }
-        cores <- get.coresOption()
-        if (is.null(cores)) {
-        parallel <- "no"
-        } else {
-            parallel <- ifelse (get.mcOption(), "multicore", "snow")
-        }
-        ncpus <- ifelse(is.null(cores), 1L, cores)
-        cl <- NULL
-        if (parallel == "snow") {
-            cl <- get.ClusterOption()
-            if (is.null(cl)) {
-                parallel <- "no"
-                warning("no cluster in ClusterOption, parallel set to no")
-            }
-        }
+        p_setup <- parallel_setup(NULL)
+        parallel <- p_setup$parallel
+        ncpus <- p_setup$ncpus
+        cl <- p_setup$cl
         res <- boot(z, statistic=EBI_boot, R=nsim,
             sim="permutation", listw=listw, nn=m, S0=S0,
             zero.policy=zero.policy,
@@ -90,6 +82,8 @@ EBImoran.mc <- function (n, x, listw, nsim, zero.policy = NULL,
         pval <- punif((diff + 1)/(nsim + 1), lower.tail=FALSE)
     else if (alternative == "greater") 
         pval <- punif((diff + 1)/(nsim + 1))
+    else pval <- punif(abs(zrank - (nsim+1)/2)/(nsim + 1), 0, 0.5,
+        lower.tail=FALSE)
     if (!is.finite(pval) || pval < 0 || pval > 1) 
 	warning("Out-of-range p-value: reconsider test arguments")
     statistic <- res[nsim + 1]
