@@ -1,6 +1,6 @@
 # Copyright (c) 2007-2008 Markus Reder and Roger Bivand
 
-lm.morantest.exact <- function(model, listw, zero.policy = NULL, 
+lm.morantest.exact <- function(model, listw, zero.policy = attr(listw, "zero.policy"), 
     alternative = "greater", spChk=NULL, resfun=weighted.residuals, 
     zero.tol=1.0e-7, Omega=NULL, save.M=NULL, save.U=NULL, useTP=FALSE,
     truncErr=1e-6, zeroTreat=0.1) 
